@@ -18,7 +18,7 @@
 from __future__ import annotations
 
 import os
-from contextlib import closing
+from contextlib import closing, contextmanager
 from functools import wraps
 from io import StringIO
 from pathlib import Path
@@ -32,7 +32,7 @@ from snowflake.sqlalchemy import URL
 from sqlalchemy import create_engine
 
 from airflow import AirflowException
-from airflow.providers.common.sql.hooks.sql import DbApiHook
+from airflow.providers.common.sql.hooks.sql import DbApiHook, return_single_query_results
 from airflow.utils.strings import to_boolean
 
 
@@ -42,12 +42,9 @@ def _try_to_boolean(value: Any):
     return value
 
 
+# TODO: Remove this when provider min airflow version >= 2.5.0 since this is
+# handled by provider manager from that version.
 def _ensure_prefixes(conn_type):
-    """
-    Remove when provider min airflow version >= 2.5.0 since this is handled by
-    provider manager from that version.
-    """
-
     def dec(func):
         @wraps(func)
         def inner():
@@ -71,10 +68,9 @@ def _ensure_prefixes(conn_type):
 
 
 class SnowflakeHook(DbApiHook):
-    """
-    A client to interact with Snowflake.
+    """A client to interact with Snowflake.
 
-    This hook requires the snowflake_conn_id connection. The snowflake host, login,
+    This hook requires the snowflake_conn_id connection. The snowflake account, login,
     and, password field must be setup in the connection. Other inputs can be defined
     in the connection or hook instantiation.
 
@@ -116,7 +112,7 @@ class SnowflakeHook(DbApiHook):
 
     @staticmethod
     def get_connection_form_widgets() -> dict[str, Any]:
-        """Returns connection widgets to add to connection form"""
+        """Returns connection widgets to add to connection form."""
         from flask_appbuilder.fieldwidgets import BS3TextAreaFieldWidget, BS3TextFieldWidget
         from flask_babel import lazy_gettext
         from wtforms import BooleanField, StringField
@@ -139,11 +135,11 @@ class SnowflakeHook(DbApiHook):
     @staticmethod
     @_ensure_prefixes(conn_type="snowflake")
     def get_ui_field_behaviour() -> dict[str, Any]:
-        """Returns custom field behaviour"""
+        """Returns custom field behaviour."""
         import json
 
         return {
-            "hidden_fields": ["port"],
+            "hidden_fields": ["port", "host"],
             "relabeling": {},
             "placeholders": {
                 "extra": json.dumps(
@@ -201,9 +197,9 @@ class SnowflakeHook(DbApiHook):
         return extra_dict.get(backcompat_key) or None
 
     def _get_conn_params(self) -> dict[str, str | None]:
-        """
-        One method to fetch connection params as a dict
-        used in get_uri() and get_connection()
+        """Fetch connection params as a dict.
+
+        This is used in ``get_uri()`` and ``get_connection()``.
         """
         conn = self.get_connection(self.snowflake_conn_id)  # type: ignore[attr-defined]
         extra_dict = conn.extra_dejson
@@ -252,7 +248,12 @@ class SnowflakeHook(DbApiHook):
                 "Please remove one."
             )
         elif private_key_file:
-            private_key_pem = Path(private_key_file).read_bytes()
+            private_key_file_path = Path(private_key_file)
+            if not private_key_file_path.is_file() or private_key_file_path.stat().st_size == 0:
+                raise ValueError("The private_key_file path points to an empty or invalid file.")
+            if private_key_file_path.stat().st_size > 4096:
+                raise ValueError("The private_key_file size is too big. Please keep it less than 4 KB.")
+            private_key_pem = Path(private_key_file_path).read_bytes()
         elif private_key_content:
             private_key_pem = private_key_content.encode()
 
@@ -277,7 +278,7 @@ class SnowflakeHook(DbApiHook):
         return conn_config
 
     def get_uri(self) -> str:
-        """Override DbApiHook get_uri method for get_sqlalchemy_engine()"""
+        """Override DbApiHook get_uri method for get_sqlalchemy_engine()."""
         conn_params = self._get_conn_params()
         return self._conn_params_to_sqlalchemy_uri(conn_params)
 
@@ -291,14 +292,13 @@ class SnowflakeHook(DbApiHook):
         )
 
     def get_conn(self) -> SnowflakeConnection:
-        """Returns a snowflake.connection object"""
+        """Returns a snowflake.connection object."""
         conn_config = self._get_conn_params()
         conn = connector.connect(**conn_config)
         return conn
 
     def get_sqlalchemy_engine(self, engine_kwargs=None):
-        """
-        Get an sqlalchemy_engine object.
+        """Get an sqlalchemy_engine object.
 
         :param engine_kwargs: Kwargs used in :func:`~sqlalchemy.create_engine`.
         :return: the created engine.
@@ -329,52 +329,67 @@ class SnowflakeHook(DbApiHook):
         handler: Callable | None = None,
         split_statements: bool = True,
         return_last: bool = True,
+        return_dictionaries: bool = False,
     ) -> Any | list[Any] | None:
-        """
-        Runs a command or a list of commands. Pass a list of sql
-        statements to the sql parameter to get them to execute
-        sequentially. The variable execution_info is returned so that
-        it can be used in the Operators to modify the behavior
-        depending on the result of the query (i.e fail the operator
-        if the copy has processed 0 files)
+        """Runs a command or a list of commands.
 
-        :param sql: the sql string to be executed with possibly multiple statements,
-          or a list of sql statements to execute
+        Pass a list of SQL statements to the SQL parameter to get them to
+        execute sequentially. The variable ``execution_info`` is returned so
+        that it can be used in the Operators to modify the behavior depending on
+        the result of the query (i.e fail the operator if the copy has processed
+        0 files).
+
+        :param sql: The SQL string to be executed with possibly multiple
+            statements, or a list of sql statements to execute
         :param autocommit: What to set the connection's autocommit setting to
             before executing the query.
         :param parameters: The parameters to render the SQL query with.
-        :param handler: The result handler which is called with the result of each statement.
-        :param split_statements: Whether to split a single SQL string into statements and run separately
-        :param return_last: Whether to return result for only last statement or for all after split
-        :return: return only result of the LAST SQL expression if handler was provided.
+        :param handler: The result handler which is called with the result of
+            each statement.
+        :param split_statements: Whether to split a single SQL string into
+            statements and run separately
+        :param return_last: Whether to return result for only last statement or
+            for all after split.
+        :param return_dictionaries: Whether to return dictionaries rather than
+            regular DBAPI sequences as rows in the result. The dictionaries are
+            of form ``{ 'column1_name': value1, 'column2_name': value2 ... }``.
+        :return: Result of the last SQL statement if *handler* is set.
+            *None* otherwise.
         """
         self.query_ids = []
 
-        scalar_return_last = isinstance(sql, str) and return_last
         if isinstance(sql, str):
             if split_statements:
                 split_statements_tuple = util_text.split_statements(StringIO(sql))
-                sql = [sql_string for sql_string, _ in split_statements_tuple if sql_string]
+                sql_list: Iterable[str] = [
+                    sql_string for sql_string, _ in split_statements_tuple if sql_string
+                ]
             else:
-                sql = [self.strip_sql_string(sql)]
+                sql_list = [self.strip_sql_string(sql)]
+        else:
+            sql_list = sql
 
-        if sql:
-            self.log.debug("Executing following statements against Snowflake DB: %s", list(sql))
+        if sql_list:
+            self.log.debug("Executing following statements against Snowflake DB: %s", sql_list)
         else:
             raise ValueError("List of SQL statements is empty")
 
         with closing(self.get_conn()) as conn:
             self.set_autocommit(conn, autocommit)
 
-            # SnowflakeCursor does not extend ContextManager, so we have to ignore mypy error here
-            with closing(conn.cursor(DictCursor)) as cur:  # type: ignore[type-var]
+            with self._get_cursor(conn, return_dictionaries) as cur:
                 results = []
-                for sql_statement in sql:
+                for sql_statement in sql_list:
                     self._run_command(cur, sql_statement, parameters)
 
                     if handler is not None:
                         result = handler(cur)
-                        results.append(result)
+                        if return_single_query_results(sql, return_last, split_statements):
+                            _last_result = result
+                            _last_description = cur.description
+                        else:
+                            results.append(result)
+                            self.descriptions.append(cur.description)
 
                     query_id = cur.sfqid
                     self.log.info("Rows affected: %s", cur.rowcount)
@@ -387,7 +402,21 @@ class SnowflakeHook(DbApiHook):
 
         if handler is None:
             return None
-        elif scalar_return_last:
-            return results[-1]
+        if return_single_query_results(sql, return_last, split_statements):
+            self.descriptions = [_last_description]
+            return _last_result
         else:
             return results
+
+    @contextmanager
+    def _get_cursor(self, conn: Any, return_dictionaries: bool):
+        cursor = None
+        try:
+            if return_dictionaries:
+                cursor = conn.cursor(DictCursor)
+            else:
+                cursor = conn.cursor()
+            yield cursor
+        finally:
+            if cursor is not None:
+                cursor.close()
