@@ -19,17 +19,20 @@ from __future__ import annotations
 import warnings
 from typing import Iterable, Mapping, Sequence
 
+from airflow.exceptions import AirflowProviderDeprecationWarning
 from airflow.providers.slack.transfers.sql_to_slack import SqlToSlackOperator
 
 
 class SnowflakeToSlackOperator(SqlToSlackOperator):
     """
-    Executes an SQL statement in Snowflake and sends the results to Slack. The results of the query are
-    rendered into the 'slack_message' parameter as a Pandas dataframe using a JINJA variable called '{{
-    results_df }}'. The 'results_df' variable name can be changed by specifying a different
-    'results_df_name' parameter. The Tabulate library is added to the JINJA environment as a filter to
-    allow the dataframe to be rendered nicely. For example, set 'slack_message' to {{ results_df |
-    tabulate(tablefmt="pretty", headers="keys") }} to send the results to Slack as an ascii rendered table.
+    Executes an SQL statement in Snowflake and sends the results to Slack.
+
+    The results of the query are rendered into the 'slack_message' parameter as a Pandas dataframe
+    using a Jinja variable called '{{ results_df }}'. The 'results_df' variable name can be changed
+    by specifying a different 'results_df_name' parameter. The Tabulate library is added to the
+    Jinja environment as a filter to allow the dataframe to be rendered nicely. For example, set
+    'slack_message' to {{ results_df | tabulate(tablefmt="pretty", headers="keys") }} to send the
+    results to Slack as an ASCII rendered table.
 
     .. seealso::
         For more information on how to use this operator, take a look at the guide:
@@ -52,8 +55,8 @@ class SnowflakeToSlackOperator(SqlToSlackOperator):
         'webhook_token' attribute needs to be specified in the 'Extra' JSON field against the slack_conn_id.
     """
 
-    template_fields: Sequence[str] = ('sql', 'slack_message')
-    template_ext: Sequence[str] = ('.sql', '.jinja', '.j2')
+    template_fields: Sequence[str] = ("sql", "slack_message")
+    template_ext: Sequence[str] = (".sql", ".jinja", ".j2")
     template_fields_renderers = {"sql": "sql", "slack_message": "jinja"}
     times_rendered = 0
 
@@ -62,9 +65,9 @@ class SnowflakeToSlackOperator(SqlToSlackOperator):
         *,
         sql: str,
         slack_message: str,
-        snowflake_conn_id: str = 'snowflake_default',
-        slack_conn_id: str = 'slack_default',
-        results_df_name: str = 'results_df',
+        snowflake_conn_id: str = "snowflake_default",
+        slack_conn_id: str = "slack_default",
+        results_df_name: str = "results_df",
         parameters: Iterable | Mapping | None = None,
         warehouse: str | None = None,
         database: str | None = None,
@@ -90,7 +93,7 @@ class SnowflakeToSlackOperator(SqlToSlackOperator):
             SnowflakeToSlackOperator is deprecated.
             Please use `airflow.providers.slack.transfers.sql_to_slack.SqlToSlackOperator`.
             """,
-            DeprecationWarning,
+            AirflowProviderDeprecationWarning,
             stacklevel=2,
         )
 
